; Name: RBTree
; Author: Kevin Jasik (CSHW89)
; Date: 28.12.11
; Description: Implementierung eines Rot-Schwarz-Baums

EnableExplicit

IncludeFile "Iterator.pb"
IncludeFile "Comparator.pb"


#RBTreeColorBlack = 0
#RBTreeColorRed = 1

Structure RBTreeNodeObj
  *value
  *parent.RBTreeNodeObj
  *left.RBTreeNodeObj
  *right.RBTreeNodeObj
  color.i
EndStructure

Structure RBTreeIteratorObj
  *VTable
  *tree.RBTreeObj
  *last.RBTreeNodeObj
  *node.RBTreeNodeObj
EndStructure


Interface RBTree
  insert.i(*value)
  delete.i(*value)
  contains.i(*value)
  getValue.i(*value)
  size.i()
  isEmpty.i()
  iterator.i()
  copy(*tree.RBTree)
  clear()
  free()
EndInterface

Structure RBTreeObj
  *VTable
  comparator.ComparatorFunction
  size.i
  *root.RBTreeNodeObj
EndStructure


Procedure RBTree_prGetParent(*this.RBTreeNodeObj)
  If (*this <> #Null)
    ProcedureReturn *this\parent
  EndIf
  ProcedureReturn #Null
EndProcedure

Procedure RBTree_prGetColor(*this.RBTreeNodeObj)
  If (*this <> #Null)
    ProcedureReturn *this\color
  EndIf
  ProcedureReturn #RBTreeColorBlack
EndProcedure

Procedure RBTree_prRotateLeft(*this.RBTreeObj, *node.RBTreeNodeObj)
  Protected.RBTreeNodeObj *newparent
  
  If (*node <> #Null)
    *newparent = *node\right
    *node\right = *newparent\left
    If (*node\right <> #Null)
      *node\right\parent = *node
    EndIf
    *newparent\parent = *node\parent
    If (*node\parent = #Null)
      *this\root = *newparent
    ElseIf (*node = *node\parent\left)
      *newparent\parent\left = *newparent
    Else
      *newparent\parent\right = *newparent
    EndIf
    *newparent\left = *node
    *node\parent = *newparent
  EndIf
EndProcedure

Procedure RBTree_prRotateRight(*this.RBTreeObj, *node.RBTreeNodeObj)
  Protected.RBTreeNodeObj *newparent
  
  If (*node <> #Null)
    *newparent = *node\left
    *node\left = *newparent\right
    If (*node\left <> #Null)
      *node\left\parent = *node
    EndIf
    *newparent\parent = *node\parent
    If (*node\parent = #Null)
      *this\root = *newparent
    ElseIf (*node = *node\parent\left)
      *newparent\parent\left = *newparent
    Else
      *newparent\parent\right = *newparent
    EndIf
    *newparent\right = *node
    *node\parent = *newparent
  EndIf
EndProcedure

Procedure RBTree_prSuccessor(*this.RBTreeObj, *node.RBTreeNodeObj)
  Protected.RBTreeNodeObj *suc, *last
  
  If (*node\right <> #Null)
    *suc = *node\right
    While (*suc\left <> #Null)
      *suc = *suc\left
    Wend
  Else
    *last = *node
    *suc  = *node\parent
    While (*suc <> #Null) And (*suc\left <> *last)
      *last = *suc
      *suc  = *suc\parent
    Wend
  EndIf
  
  ProcedureReturn *suc
EndProcedure


Procedure RBTree_prFixDel(*this.RBTreeObj, *node.RBTreeNodeObj)
  Protected.RBTreeNodeObj *brother
  Protected *value
  
  While (*node <> *this\root) And (*node\color = #RBTreeColorBlack)
    If (*node = *node\parent\left)
      
      *brother = *node\parent\right
      If (*brother\color = #RBTreeColorRed)
        *brother\color = #RBTreeColorBlack
        *node\parent\color = #RBTreeColorRed
        RBTree_prRotateLeft(*this, *node\parent)
        *brother = *node\parent\right
      EndIf
      If (RBTree_prGetColor(*brother\left) = #RBTreeColorBlack) And (RBTree_prGetColor(*brother\right) = #RBTreeColorBlack)
        *brother\color = #RBTreeColorRed
        *node = *node\parent
      Else
        If (RBTree_prGetColor(*brother\right) = #RBTreeColorBlack)
          *brother\left\color = #RBTreeColorBlack
          *brother\color = #RBTreeColorRed
          RBTree_prRotateRight(*this, *brother)
          *brother = *node\parent\right
        EndIf
        *brother\color = *node\parent\color
        *node\parent\color   = #RBTreeColorBlack
        *brother\right\color = #RBTreeColorBlack
        RBTree_prRotateLeft(*this, *node\parent)
        *node = *this\root
      EndIf
      
    Else
      
      *brother = *node\parent\left
      If (*brother\color = #RBTreeColorRed)
        *brother\color = #RBTreeColorBlack
        *node\parent\color = #RBTreeColorRed
        RBTree_prRotateRight(*this, *node\parent)
        *brother = *node\parent\left
      EndIf
      If (RBTree_prGetColor(*brother\left) = #RBTreeColorBlack) And (RBTree_prGetColor(*brother\right) = #RBTreeColorBlack)
        *brother\color = #RBTreeColorRed
        *node = *node\parent
      Else
        If (RBTree_prGetColor(*brother\left) = #RBTreeColorBlack)
          *brother\right\color = #RBTreeColorBlack
          *brother\color = #RBTreeColorRed
          RBTree_prRotateLeft(*this, *brother)
          *brother = *node\parent\left
        EndIf
        *brother\color = *node\parent\color
        *node\parent\color  = #RBTreeColorBlack
        *brother\left\color = #RBTreeColorBlack
        RBTree_prRotateRight(*this, *node\parent)
        *node = *this\root
      EndIf
      
    EndIf
  Wend
  *node\color = #RBTreeColorBlack
EndProcedure

Procedure RBTree_prDelNode(*this.RBTreeObj, *node.RBTreeNodeObj)
  Protected.RBTreeNodeObj *rep
  
  If (*node\left <> #Null) And (*node\right <> #Null)
    *rep = *node\left
    While (*rep\right <> #Null)
      *rep = *rep\right
    Wend
    *node\value = *rep\value
    *node = *rep
  EndIf
  
  If (*node\parent = #Null)
    *this\root = #Null
  ElseIf (*node\left = #Null) And (*node\right = #Null)
    If (*node\color = #RBTreeColorBlack)
      RBTree_prFixDel(*this, *node)
    EndIf
    If (*node\parent <> #Null)
      If (*node = *node\parent\left)
        *node\parent\left = #Null
      Else
        *node\parent\right = #Null
      EndIf
    EndIf
  Else
    If (*node\left <> #Null)
      *rep = *node\left
    Else
      *rep = *node\right
    EndIf
    *rep\parent = *node\parent
    If (*node\parent = #Null)
      *this\root = *rep
    ElseIf (*node\parent\left = *node)
      *node\parent\left = *rep
    Else
      *node\parent\right = *rep
    EndIf
    If (*node\color = #RBTreeColorBlack)
      RBTree_prFixDel(*this, *rep)
    EndIf
  EndIf
  
  FreeMemory(*node)
  *this\size - 1
EndProcedure


Procedure.i RBTree_insert(*this.RBTreeObj, *value)
  Protected.RBTreeNodeObj *node, *parent
  Protected cmp.i
  
  *parent = #Null
  *node = *this\root
  While (*node <> #Null)
    *parent = *node
    cmp = *this\comparator(*value, *node\value)
    If (cmp < 0)
      *node = *node\left
    ElseIf (cmp > 0)
      *node = *node\right
    Else
      ProcedureReturn #False
    EndIf
  Wend
  
  *node = AllocateMemory(SizeOf(RBTreeNodeObj))
  *node\value = *value
  *node\color = #RBTreeColorRed
  *node\parent = *parent
  If (cmp < 0)
    *parent\left = *node
  ElseIf (cmp > 0)
    *parent\right = *node
  Else
    *this\root = *node
  EndIf
  *this\size + 1
  
  While (RBTree_prGetParent(*node) <> #Null) And (*node\parent\color = #RBTreeColorRed)
    *parent = *node\parent
    If (*parent\parent\left = *parent)
      
      If (RBTree_prGetColor(*parent\parent\right) = #RBTreeColorRed)
        *parent\parent\left\color  = #RBTreeColorBlack
        *parent\parent\right\color = #RBTreeColorBlack
        *parent\parent\color = #RBTreeColorRed
        *node = *parent\parent
      Else
        If (*parent\right = *node)
          RBTree_prRotateLeft(*this, *parent)
          Swap *parent, *node
        EndIf
        RBTree_prRotateRight(*this, *parent\parent)
        *parent\color = #RBTreeColorBlack
        *parent\right\color = #RBTreeColorRed
      EndIf
      
    Else
      
      If (RBTree_prGetColor(*parent\parent\left) = #RBTreeColorRed)
        *parent\parent\left\color  = #RBTreeColorBlack
        *parent\parent\right\color = #RBTreeColorBlack
        *parent\parent\color = #RBTreeColorRed
        *node = *parent\parent
      Else
        If (*parent\left = *node)
          RBTree_prRotateRight(*this, *parent)
          Swap *parent, *node
        EndIf
        RBTree_prRotateLeft(*this, *parent\parent)
        *parent\color = #RBTreeColorBlack
        *parent\left\color = #RBTreeColorRed
      EndIf
      
    EndIf
  Wend
  
  *this\root\color = #RBTreeColorBlack
  ProcedureReturn #True
EndProcedure


Procedure.i RBTree_delete(*this.RBTreeObj, *value)
  Protected.RBTreeNodeObj *node
  Protected cmp.i
  
  *node = *this\root
  While (*node <> #Null)
    cmp = *this\comparator(*value, *node\value)
    If (cmp < 0)
      *node = *node\left
    ElseIf (cmp > 0)
      *node = *node\right
    Else
      Break
    EndIf
  Wend
  
  If (*node <> #Null)
    RBTree_prDelNode(*this, *node)
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure


Procedure.i RBTree_contains(*this.RBTreeObj, *value)
  Protected.RBTreeNodeObj *node
  Protected cmp.i
  
  *node = *this\root
  While (*node <> #Null)
    cmp = *this\comparator(*value, *node\value)
    If (cmp < 0)
      *node = *node\left
    ElseIf (cmp > 0)
      *node = *node\right
    Else
      ProcedureReturn #True
    EndIf
  Wend
  ProcedureReturn #False
EndProcedure

Procedure.i RBTree_getValue(*this.RBTreeObj, *value)
  Protected.RBTreeNodeObj *node
  Protected cmp.i
  
  *node = *this\root
  While (*node <> #Null)
    cmp = *this\comparator(*value, *node\value)
    If (cmp < 0)
      *node = *node\left
    ElseIf (cmp > 0)
      *node = *node\right
    Else
      ProcedureReturn *node\value
    EndIf
  Wend
  ProcedureReturn #Null
EndProcedure


Procedure.i RBTree_size(*this.RBTreeObj)
  ProcedureReturn *this\size
EndProcedure

Procedure.i RBTree_isEmpty(*this.RBTreeObj)
  If (*this\size = 0)
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure


Procedure.i RBTreeIterator_hasNext(*this.RBTreeIteratorObj)
  If (*this\node <> #Null)
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure.i RBTreeIterator_nextEl(*this.RBTreeIteratorObj)
  If (*this\node <> #Null)
    *this\last = *this\node
    *this\node = RBTree_prSuccessor(*this\tree, *this\node)
  EndIf
  
  ProcedureReturn *this\last\value
EndProcedure

Procedure.i RBTreeIterator_delete(*this.RBTreeIteratorObj)
  If (*this\last <> #Null)
    RBTree_prDelNode(*this\tree, *this\last)
    *this\last = #Null
    ProcedureReturn #True
  EndIf
  ProcedureReturn #False
EndProcedure

Procedure RBTreeIterator_free(*this.RBTreeIteratorObj)
  FreeMemory(*this)
EndProcedure


Procedure RBTree_iterator(*this.RBTreeObj)
  Protected *iterator.RBTreeIteratorObj
  
  *iterator = AllocateMemory(SizeOf(RBTreeIteratorObj))
  *iterator\VTable = ?RBTreeIteratorFunctions
  *iterator\tree = *this
  
  If (*this\root <> #Null)
    *iterator\node = *this\root
    While (*iterator\node\left <> #Null)
      *iterator\node = *iterator\node\left
    Wend
  EndIf
  
  ProcedureReturn *iterator
  
  DataSection
    RBTreeIteratorFunctions:
    Data.i @RBTreeIterator_hasNext()
    Data.i @RBTreeIterator_nextEl()
    Data.i @RBTreeIterator_delete()
    Data.i @RBTreeIterator_free()
  EndDataSection
EndProcedure


Procedure RBTree_prClearNode(*node.RBTreeNodeObj)
  If (*node <> #Null)
    RBTree_prClearNode(*node\left)
    RBTree_prClearNode(*node\right)
    FreeMemory(*node)
  EndIf
EndProcedure

Procedure RBTree_clear(*this.RBTreeObj)
  RBTree_prClearNode(*this\root)
  *this\root = #Null
  *this\size = 0
EndProcedure

Procedure RBTree_free(*this.RBTreeObj)
  RBTree_clear(*this)
  FreeMemory(*this)
EndProcedure


Procedure RBTree_prCopyNode(*node.RBTreeNodeObj, *parent.RBTreeNodeObj)
  Protected *copy.RBTreeNodeObj
  
  If (*node <> #Null)
    *copy = AllocateMemory(SizeOf(RBTreeNodeObj))
    *copy\parent = *parent
    *copy\value = *node\value
    *copy\color = *node\color
    *copy\left  = RBTree_prCopyNode(*node\left,  *copy)
    *copy\right = RBTree_prCopyNode(*node\right, *copy)
  EndIf
  
  ProcedureReturn *copy
EndProcedure

Procedure RBTree_copy(*this.RBTreeObj, *tree.RBTreeObj)
  RBTree_clear(*tree)
  *tree\comparator = *this\comparator
  *tree\size = *this\size
  *tree\root = RBTree_prCopyNode(*this\root, #Null)
EndProcedure


Procedure NewRBTree(comparator.ComparatorFunction=#Null)
  Protected *this.RBTreeObj
  
  *this = AllocateMemory(SizeOf(RBTreeObj))
  *this\VTable = ?RBTreeFunctions
  *this\comparator = comparator
  If (*this\comparator = #Null)
    *this\comparator = @StandardComparatorFunction()
  EndIf
  
  ProcedureReturn *this
  
  DataSection
    RBTreeFunctions:
    Data.i @RBTree_insert()
    Data.i @RBTree_delete()
    Data.i @RBTree_contains()
    Data.i @RBTree_getValue()
    Data.i @RBTree_size()
    Data.i @RBTree_isEmpty()
    Data.i @RBTree_iterator()
    Data.i @RBTree_copy()
    Data.i @RBTree_clear()
    Data.i @RBTree_free()
  EndDataSection
EndProcedure



; IDE Options = PureBasic 4.60 (Windows - x86)